@@ -28,8 +28,7 @@
 unsigned int cdnsp_port_speed(unsigned int port_status)
 {
 	/*Detect gadget speed based on PORTSC register*/
-	if (DEV_SUPERSPEEDPLUS(port_status) ||
-	    DEV_SSP_GEN1x2(port_status) || DEV_SSP_GEN2x2(port_status))
+	if (DEV_SUPERSPEEDPLUS(port_status))
 		return USB_SPEED_SUPER_PLUS;
 	else if (DEV_SUPERSPEED(port_status))
 		return USB_SPEED_SUPER;
@@ -139,26 +138,6 @@ static void cdnsp_clear_port_change_bit(struct cdnsp_device *pdev,
 	       (portsc & PORT_CHANGE_BITS), port_regs);
 }
 
-static void cdnsp_set_apb_timeout_value(struct cdnsp_device *pdev)
-{
-	struct cdns *cdns = dev_get_drvdata(pdev->dev);
-	__le32 __iomem *reg;
-	void __iomem *base;
-	u32 offset = 0;
-	u32 val;
-
-	if (!cdns->override_apb_timeout)
-		return;
-
-	base = &pdev->cap_regs->hc_capbase;
-	offset = cdnsp_find_next_ext_cap(base, offset, D_XEC_PRE_REGS_CAP);
-	reg = base + offset + REG_CHICKEN_BITS_3_OFFSET;
-
-	val  = le32_to_cpu(readl(reg));
-	val = CHICKEN_APB_TIMEOUT_SET(val, cdns->override_apb_timeout);
-	writel(cpu_to_le32(val), reg);
-}
-
 static void cdnsp_set_chicken_bits_2(struct cdnsp_device *pdev, u32 bit)
 {
 	__le32 __iomem *reg;
@@ -399,7 +378,7 @@ int cdnsp_ep_enqueue(struct cdnsp_ep *pep, struct cdnsp_request *preq)
 		ret = cdnsp_queue_bulk_tx(pdev, preq);
 		break;
 	case USB_ENDPOINT_XFER_ISOC:
-		ret = cdnsp_queue_isoc_tx_prepare(pdev, preq);
+		ret = cdnsp_queue_isoc_tx(pdev, preq);
 	}
 
 	if (ret)
@@ -547,7 +526,6 @@ int cdnsp_wait_for_cmd_compl(struct cdnsp_device *pdev)
 	dma_addr_t cmd_deq_dma;
 	union cdnsp_trb *event;
 	u32 cycle_state;
-	u32 retry = 10;
 	int ret, val;
 	u64 cmd_dma;
 	u32  flags;
@@ -579,23 +557,8 @@ int cdnsp_wait_for_cmd_compl(struct cdnsp_device *pdev)
 		flags = le32_to_cpu(event->event_cmd.flags);
 
 		/* Check the owner of the TRB. */
-		if ((flags & TRB_CYCLE) != cycle_state) {
-			/*
-			 * Give some extra time to get chance controller
-			 * to finish command before returning error code.
-			 * Checking CMD_RING_BUSY is not sufficient because
-			 * this bit is cleared to '0' when the Command
-			 * Descriptor has been executed by controller
-			 * and not when command completion event has
-			 * be added to event ring.
-			 */
-			if (retry--) {
-				udelay(20);
-				continue;
-			}
-
+		if ((flags & TRB_CYCLE) != cycle_state)
 			return -EINVAL;
-		}
 
 		cmd_dma = le64_to_cpu(event->event_cmd.cmd_trb);
 
@@ -1813,8 +1776,6 @@ static void cdnsp_get_rev_cap(struct cdnsp_device *pdev)
 	reg += cdnsp_find_next_ext_cap(reg, 0, RTL_REV_CAP);
 	pdev->rev_cap  = reg;
 
-	pdev->rtl_revision = readl(&pdev->rev_cap->rtl_revision);
-
 	dev_info(pdev->dev, "Rev: %08x/%08x, eps: %08x, buff: %08x/%08x\n",
 		 readl(&pdev->rev_cap->ctrl_revision),
 		 readl(&pdev->rev_cap->rtl_revision),
@@ -1840,15 +1801,6 @@ static int cdnsp_gen_setup(struct cdnsp_device *pdev)
 	pdev->hci_version = HC_VERSION(pdev->hcc_params);
 	pdev->hcc_params = readl(&pdev->cap_regs->hcc_params);
 
-	/*
-	 * Override the APB timeout value to give the controller more time for
-	 * enabling UTMI clock and synchronizing APB and UTMI clock domains.
-	 * This fix is platform specific and is required to fixes issue with
-	 * reading incorrect value from PORTSC register after resuming
-	 * from L1 state.
-	 */
-	cdnsp_set_apb_timeout_value(pdev);
-
 	cdnsp_get_rev_cap(pdev);
 
 	/* Make sure the Device Controller is halted. */
